package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/go-git/go-git/v5"
	"github.com/go-git/go-git/v5/plumbing"
	"strings"
)

type BranchActions struct {
	repository *git.Repository
}

func (b *BranchActions) ExecuteBranchAction() (err error) {
	switch commandType {
	case CreateBranch:
		var createBranchCommandConfig CreateBranchCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &createBranchCommandConfig)
		err = b.Create(createBranchCommandConfig)
	case DeleteBranch:
		var config DeleteBranchCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		err = b.Delete(config)
	case Checkout:
		var config CheckoutCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		err = b.Checkout(config.BranchName, config.Commit, config.Force)
	default:
		err = errors.New("Unknown branch action " + commandType)
	}

	if updateHopsworks {
		message := fmt.Sprintf("%s was successful", commandType)
		var finalState = SuccessState
		if err != nil {
			message = fmt.Sprintf("%s was failed. %s", commandType, err.Error())
			logger.LogError(err.Error(), nil)
			finalState = FailedState
		} else {
			logger.LogInfo(message, nil)
			UpdateBranchCommits(b.repository)
		}
		UpdateFinalRepositoryState(b.repository, message, finalState)
	}
	return err
}

func (b *BranchActions) Create(createBranchCommandConfig CreateBranchCommandConfiguration) error {
	branchName := createBranchCommandConfig.BranchName
	logger.LogInfo(fmt.Sprintf("Creating new branch: %s", branchName), nil)
	headRef, err := b.repository.Head()
	if err != nil {
		logger.LogError(err.Error(), nil)
		return err
	}
	// check if branch exist and return proper error
	exist, _ := b.CheckIfBranchExits(branchName)
	if exist {
		logger.LogError(fmt.Sprintf("Error on creating branch: %s. %s", branchName, git.ErrBranchExists.Error()), nil)
		return git.ErrBranchExists
	}
	branch := plumbing.ReferenceName(fmt.Sprintf("refs/heads/%s", branchName))
	// Create a new plumbing.HashReference object with the name of the branch
	// and the hash from the HEAD
	ref := plumbing.NewHashReference(branch, headRef.Hash())
	// The created reference is saved in the storage.
	err = b.repository.Storer.SetReference(ref)
	if err != nil {
		logger.LogError(fmt.Sprintf("Failed to create branch %s. %s", branchName, err.Error()), nil)
		return err
	}
	// get commits for the just created branch
	if updateHopsworks && !createBranchCommandConfig.Checkout {
		repoActions := &RepositoryActions{r: b.repository}
		commits, _ := repoActions.GetCommits()
		SendBranchCommitsToHopsworks(branchName, commits)
	}
	if createBranchCommandConfig.Checkout {
		return b.Checkout(createBranchCommandConfig.BranchName, "", false)
	}
	return nil
}

func (b *BranchActions) Delete(deleteBranchConfiguration DeleteBranchCommandConfiguration) error {
	branchName := deleteBranchConfiguration.BranchName
	// check if branch exist and return proper error
	exist, err := b.CheckIfBranchExits(branchName)
	if !exist {
		logger.LogError(fmt.Sprintf("Error on deleting branch: %s. %s", branchName, git.ErrBranchNotFound.Error()), nil)
		return git.ErrBranchNotFound
	}
	branch := plumbing.NewBranchReferenceName(branchName)
	logger.LogInfo(fmt.Sprintf("git branch -D %s", branchName), nil)
	err = b.repository.Storer.RemoveReference(branch)
	return err
}

func (b *BranchActions) Checkout(branchName string, commit string, force bool) error {
	w, err := b.repository.Worktree()
	if err != nil {
		return err
	}
	checkoutOptions := &git.CheckoutOptions{
		Force: force,
	}
	//Either a commit or branch should be provided but not both
	if branchName != "" {
		logger.LogInfo(fmt.Sprintf("Checking out to branch %s", branchName), nil)
		branch := fmt.Sprintf("refs/heads/%s", branchName)
		b := plumbing.ReferenceName(branch)
		checkoutOptions.Branch = b
	} else if commit != "" {
		checkoutOptions.Hash = plumbing.NewHash(commit)
	}
	err = w.Checkout(checkoutOptions)
	if err != nil && strings.Contains(err.Error(), "reference not found") && branchName != "" {
		logger.LogInfo("Branch was not found locally, trying checking out to remote branch", nil)
		checkoutCommand := "git checkout " + branchName
		remoteCheckoutErr := ExecuteOnPath(repositoryPath, checkoutCommand)
		if remoteCheckoutErr != nil {
			errorMessage := remoteCheckoutErr.Error()
			if strings.Contains(errorMessage, "did not match any file(s) known to git") {
				errorMessage = fmt.Sprintf("branch `%s` does not exist", branchName)
			}
			return errors.New(errorMessage)
		}
		return nil
	}
	return err
}

func (b *BranchActions) CheckIfBranchExits(branchName string) (bool, error) {
	exist := false
	branches, err := b.repository.Branches()
	if err != nil {
		return exist, err
	}
	branches.ForEach(func(branch *plumbing.Reference) error {
		if branch.Name().Short() == branchName {
			exist = true
		}
		return nil
	})

	return exist, err
}
