package main

import (
	"github.com/go-git/go-git/v5"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"testing"
)

var testBranchName = "test_branch"

func TestBranchActions_Create(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "create_branch")
	branchActions := &BranchActions{repository: r}
	err := branchActions.Create(CreateBranchCommandConfiguration{BranchName: testBranchName})
	require.Nil(t, err)
	branchExist, err := branchActions.CheckIfBranchExits(testBranchName)
	require.Nil(t, err)
	require.Equal(t, branchExist, true)
}

func TestBranchActions_CreateAlreadyExistingBranch(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "create_branch_already_existing")
	branchActions := &BranchActions{repository: r}
	alreadyExistingBranchName := "main"
	err := branchActions.Create(CreateBranchCommandConfiguration{BranchName: alreadyExistingBranchName})
	require.NotNil(t, err)
	assert.Equal(t, git.ErrBranchExists, err)
}

func TestBranchActions_Delete(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "delete_branch")
	branchActions := &BranchActions{repository: r}
	err := branchActions.Create(CreateBranchCommandConfiguration{BranchName: testBranchName})
	require.Nil(t, err)
	branchExist, err := branchActions.CheckIfBranchExits(testBranchName)
	require.Nil(t, err)
	require.Equal(t, branchExist, true)
	err = branchActions.Delete(DeleteBranchCommandConfiguration{BranchName: testBranchName})
	require.Nil(t, err)
	branchExist, err = branchActions.CheckIfBranchExits(testBranchName)
	require.Nil(t, err)
	require.Equal(t, branchExist, false)
}

func TestBranchActions_DeleteNonExistentBranch(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "delete_branch_non_existent")
	branchActions := &BranchActions{repository: r}
	nonExistingBranchName := "xyz"
	err := branchActions.Delete(DeleteBranchCommandConfiguration{BranchName: nonExistingBranchName})
	require.NotNil(t, err)
	assert.Equal(t, git.ErrBranchNotFound, err)
}

func TestBranchActions_CheckoutOnCreate(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "create_checkout_branch")
	repoActions := &RepositoryActions{r: r}
	//get the current commit hash before checking out
	_, commitHash1, err := repoActions.GetRepositoryBranchAndCommit()
	branchActions := &BranchActions{repository: r}
	err = branchActions.Create(CreateBranchCommandConfiguration{BranchName: testBranchName, Checkout: true})
	require.Nil(t, err)
	currentBranch, commitHash2, err := repoActions.GetRepositoryBranchAndCommit()
	require.Nil(t, err)
	require.Equal(t, currentBranch, testBranchName)
	require.Equal(t, commitHash1, commitHash2)
}

func TestBranchActions_Checkout(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "checkout_branch")
	branchActions := &BranchActions{repository: r}
	err := branchActions.Create(CreateBranchCommandConfiguration{BranchName: testBranchName})
	require.Nil(t, err)
	err = branchActions.Checkout(testBranchName, "", true)
	require.Nil(t, err)
}

func TestBranchActions_CheckoutRemoteBranch(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "checkout_remote_branch")
	branchActions := &BranchActions{repository: r}
	err := branchActions.Checkout("test_remote_branch", "", true)
	require.Nil(t, err)
	repoActions := &RepositoryActions{r: r}
	currentBranch, _, err := repoActions.GetRepositoryBranchAndCommit()
	require.Nil(t, err)
	require.Equal(t, currentBranch, "test_remote_branch")
}
