package main

import (
	"encoding/json"
	"fmt"
	"github.com/go-git/go-git/v5"
	"github.com/gorilla/mux"
	"github.com/stretchr/testify/require"
	"net/http"
	"net/http/httptest"
	"os"
	"testing"
)

var cloneRepoConfig = CloneCommandConfiguration{Url: TestRepoUrl}

type GitRepository struct {
	Id             int
	CurrentBranch  string
	CurrentCommit  string
	ExecutionState string
	Commits        []CommitDTO
}

//To be used for testing if we can update repository on server
var gitRepositoryTest = &GitRepository{ExecutionState: "Initializing"}

func setup() {
	err := os.RemoveAll(TestDir)
	if err != nil {
		fmt.Println("Error in deleting test repo " + err.Error())
	}
	os.MkdirAll(TestDir, 0775)
}

func teardown() {
	err := os.RemoveAll(TestDir)
	if err != nil {
		fmt.Println("Error in deleting test repo " + err.Error())
	}
}

func TestMain(m *testing.M) {
	setup()
	m.Run()
	teardown()
}

func createRepoDir(name string) (string, error) {
	dir := TestDir + "/" + name
	err := os.Mkdir(dir, 0775)
	if err != nil {
		logger.LogError(fmt.Sprintf("failed to create directory %s: %s", name, err.Error()), nil)
	}
	return dir, err
}

func ResetTestRepo() {
	gitRepositoryTest.ExecutionState = "Initializing"
	gitRepositoryTest.CurrentBranch = ""
	gitRepositoryTest.CurrentCommit = ""
	gitRepositoryTest.Commits = []CommitDTO{}
}

func CloneRepoTest(t *testing.T, command string) (r *git.Repository, repoDir string, err error) {
	repoDir, err = createRepoDir(command)
	require.Nil(t, err)
	r, err = CloneRepo(repoDir, cloneRepoConfig)
	require.Nil(t, err)
	require.NotNil(t, r)
	repositoryPath = repoDir
	return r, repoDir, err
}

func UpdateExecutionStateHandler(writer http.ResponseWriter, request *http.Request) {
	logger.LogInfo("Test updating repo status", nil)
	var executionState GitCommandExecutionStateUpdateDTO
	err := json.NewDecoder(request.Body).Decode(&executionState)
	responseCode := http.StatusOK
	if err != nil {
		logger.LogError("Error : "+err.Error(), nil)
		responseCode = http.StatusInternalServerError
	}
	gitRepositoryTest.ExecutionState = executionState.GitOpExecutionState
	gitRepositoryTest.CurrentBranch = executionState.Branch
	gitRepositoryTest.CurrentCommit = executionState.CommitHash
	WriteFinalResponse(writer, responseCode)
}

func UpdateBranchCommitsHandler(writer http.ResponseWriter, request *http.Request) {
	logger.LogInfo("Test updating branch commits", nil)
	var branchCommits BranchCommitsDTO
	err := json.NewDecoder(request.Body).Decode(&branchCommits)
	responseCode := http.StatusOK
	if err != nil {
		logger.LogError(err.Error(), nil)
		responseCode = http.StatusInternalServerError
	}
	gitRepositoryTest.Commits = branchCommits.Commits
	WriteFinalResponse(writer, responseCode)
}

func WriteFinalResponse(writer http.ResponseWriter, responseCode int) {
	writer.WriteHeader(responseCode)
	resp, err := json.Marshal(gitRepositoryTest)
	if err != nil {
		logger.LogError(err.Error(), nil)
		writer.WriteHeader(http.StatusInternalServerError)
	} else {
		writer.WriteHeader(http.StatusOK)
	}
	writer.Write(resp)
}

func HttpMock() *httptest.Server {
	handler := mux.NewRouter()
	handler.HandleFunc("/hopsworks-api/api/project/{projectId}/git/repository/{repositoryId}/execution/{executionId}/state", UpdateExecutionStateHandler).Methods("PUT")
	handler.HandleFunc("/hopsworks-api/api/project/{projectId}/git/repository/{repositoryId}", UpdateExecutionStateHandler).Methods("PUT")
	handler.HandleFunc("/hopsworks-api/api/project/{projectId}/git/repository/{repositoryId}/branch/{branchName}/commit", UpdateBranchCommitsHandler).Methods("PUT")
	return httptest.NewServer(handler)
}
