package main

import (
	"bytes"
	"crypto/tls"
	"encoding/json"
	"fmt"
	re_tryable_http "github.com/hashicorp/go-retryablehttp"
	"io/ioutil"
	"net"
	"net/http"
	"runtime"
	"time"
)

var hopsBaseUrl string

func SendFinalStateToHopsworks(currentCommit string, currentBranch string, message string, finalState string) {
	remoteEndpointUrl := GetFullEndPoint(fmt.Sprintf("execution/%s/state", executionId))
	postBody := new(bytes.Buffer)
	dto := &GitCommandExecutionStateUpdateDTO{}
	if finalState == FailedState {
		dto = &GitCommandExecutionStateUpdateDTO{
			GitOpExecutionState: FailedState,
			Message:             message,
		}
	} else {
		dto = &GitCommandExecutionStateUpdateDTO{
			GitOpExecutionState: SuccessState,
			Message:             message,
			CommitHash:          currentCommit,
			Branch:              currentBranch,
		}
	}
	json.NewEncoder(postBody).Encode(dto)
	SendHttpReq(remoteEndpointUrl, postBody, http.MethodPut)
}

func SendBranchCommitsToHopsworks(currentBranch string, commits []CommitDTO) {
	//update branch commits
	remoteEndpointUrl := GetFullEndPoint("branch/" + currentBranch + "/commit")
	postBody := new(bytes.Buffer)
	body := &BranchCommitsDTO{
		Commits: commits,
	}
	json.NewEncoder(postBody).Encode(body)
	SendHttpReq(remoteEndpointUrl, postBody, http.MethodPut)
}

func UpdateExecutionStateToRunning() {
	remoteEndpointUrl := GetFullEndPoint(fmt.Sprintf("execution/%s/state", executionId))
	body := &GitCommandExecutionStateUpdateDTO{
		GitOpExecutionState: RunningState,
		CommitHash:          "",
		Branch:              "",
		Message:             "",
	}
	postBody := new(bytes.Buffer)
	json.NewEncoder(postBody).Encode(body)
	SendHttpReq(remoteEndpointUrl, postBody, http.MethodPut)
}

func SendHttpReq(remoteUrl string, postBody *bytes.Buffer, httpMethod string) {
	retryClient := re_tryable_http.NewClient()
	retryClient.HTTPClient = DefaultPooledClient()
	retryClient.RetryWaitMin = 2 * time.Second
	retryClient.RetryWaitMax = 4 * time.Second
	retryClient.RetryMax = 10
	standardClient := retryClient.StandardClient()
	logger.LogInfo(fmt.Sprintf("--> HTTP request to Hopsworks, URL: %s, Method: %s", remoteUrl, httpMethod), nil)
	bearer := "Bearer " + jwtToken
	req, err := http.NewRequest(httpMethod, remoteUrl, postBody)
	req.Header.Set("Authorization", bearer)
	req.Header.Add("Accept", "application/json")
	req.Header.Add("Content-Type", "application/json;charset=UTF-8")

	standardClient.CheckRedirect = func(req *http.Request, via []*http.Request) error {
		for key, val := range via[0].Header {
			req.Header[key] = val
		}
		return err
	}
	resp, err := standardClient.Do(req)
	if err != nil {
		logger.LogError(err.Error(), nil)
	} else {
		defer resp.Body.Close()
		data, _ := ioutil.ReadAll(resp.Body)
		logger.LogInfo(fmt.Sprintf("<-- Received data from Hopsworks:  %s", string(data)), nil)
	}
}

func GetFullEndPoint(endpoint string) (fullEndPoint string) {
	return "https://" + hopsBaseUrl + "/" + endpoint
}

func DefaultPooledTransport() *http.Transport {
	transport := &http.Transport{
		Proxy: http.ProxyFromEnvironment,
		DialContext: (&net.Dialer{
			Timeout:   30 * time.Second,
			KeepAlive: 30 * time.Second,
			DualStack: true,
		}).DialContext,
		MaxIdleConns:          100,
		IdleConnTimeout:       90 * time.Second,
		TLSHandshakeTimeout:   10 * time.Second,
		ExpectContinueTimeout: 1 * time.Second,
		ForceAttemptHTTP2:     true,
		MaxIdleConnsPerHost:   runtime.GOMAXPROCS(0) + 1,
		TLSClientConfig:       &tls.Config{InsecureSkipVerify: true},
	}
	return transport
}

func DefaultPooledClient() *http.Client {
	return &http.Client{
		Transport: DefaultPooledTransport(),
	}
}
