package main

import (
	"encoding/json"
	"errors"
	"flag"
	"fmt"
	"github.com/go-git/go-git/v5"
	"os"
	"strconv"
)

var commandType string
var commandConfig string
var executionId string
var projectId string
var gitUsername string
var gitToken string
var repositoryId string
var repositoryPath string
var remoteServerUrl string
var jwtToken string
var logger Logger = *InitLogger("debug", false)
var updateHopsworks bool

const successExitCode = 0
const invalidCommandJsonStringExitCode = 1
const missingCommandTypeExitCode = 2
const missingRepositoryPathExitCode = 3
const executeCommandError = 4

func main() {
	os.Exit(ExecuteCommand())
}

func ExecuteCommand() int {
	parseArgsExitCode := ParseArgs()
	if parseArgsExitCode != 0 {
		return parseArgsExitCode
	}
	logger.LogInfo(fmt.Sprintf("Starting a %s operation", commandType), nil)
	var err error
	if updateHopsworks {
		UpdateExecutionStateToRunning()
	}
	repositoryActions := &RepositoryActions{}
	branchActions := &BranchActions{}
	remoteActions := &RemoteActions{}
	fileActions := &FileActions{}
	var repo *git.Repository
	if commandType != Clone {
		repo, err = OpenRepo(repositoryPath)
		if err != nil {
			return executeCommandError
		}
	}
	switch commandType {
	case Clone:
		var config CloneCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		repo, err = CloneRepo(repositoryPath, config)
	case Commit, Pull, Push, Status:
		repositoryActions.r = repo
		err = repositoryActions.ExecuteRepositoryAction()
	case CreateBranch, DeleteBranch, Checkout:
		branchActions.repository = repo
		err = branchActions.ExecuteBranchAction()
	case AddRemote, DeleteRemote:
		remoteActions.repository = repo
		_, err = remoteActions.ExecuteRemoteAction(commandType)
	case FileCheckout:
		fileActions.repository = repo
		_, err = fileActions.ExecuteFileAction()
	default:
		err = errors.New(fmt.Sprintf("unknown command type: %s", commandType))
		SendFinalStateToHopsworks("", "", err.Error(), FailedState)
	}
	if err != nil {
		return executeCommandError
	}
	return successExitCode
}

func ParseArgs() int {
	var hopsfsMountPoint string
	var updateHopsworksArg string
	flag.StringVar(&commandType, "commandType", "", "The command to be executed. One of: clone, commit,"+
		" pull, push, create_branch, delete_branch, checkout, status, add_remote, delete_remote, file_checkout. Can also be passed as a key,with name `CommandType`, in the commandConfig")
	flag.StringVar(&commandConfig, "commandConfig", "", "json string of the command configuration to be executed")
	flag.StringVar(&repositoryPath, "repositoryPath", "", "Path to the repository folder. Can also be passed as a key,with name `path`, in the commandConfig")
	flag.StringVar(&jwtToken, "jwtToken", "", "[optional] jwt token")
	flag.StringVar(&executionId, "executionId", "", "[optional] The execution id in hopsworks")
	flag.StringVar(&repositoryId, "repositoryId", "", "[optional] The repository id in hopsworks")
	flag.StringVar(&projectId, "projectId", "", "[optional] The project id in hopsworks")
	flag.StringVar(&gitUsername, "gitUsername", "", "[optional] Git username")
	flag.StringVar(&gitToken, "gitToken", "", "[optional] Git token")
	flag.StringVar(&remoteServerUrl, "remoteServerUrl", "", "[optional] Hopsworks(glassfish) ip address")
	flag.StringVar(&hopsfsMountPoint, "hopsfsMountPoint", "", "[optional] The mounting point for hopsfs in the container if used with hopsfs-mount")
	flag.StringVar(&updateHopsworksArg, "updateHopsworks", "false", "[optional] Update hopsworks repos or not after each command execution. 'True' or 'False'. "+
		"To be able to update hopsworks provide projectId, repositoryId and executionId")

	flag.Usage = Usage
	flag.Parse()

	updateHopsworks, _ = strconv.ParseBool(updateHopsworksArg)

	if !json.Valid([]byte(commandConfig)) {
		logger.LogError(fmt.Sprintf("commandConfig:%s is not a valid json string", commandConfig), nil)
		return invalidCommandJsonStringExitCode
	}
	var baseConfig BaseCommandConfig
	json.Unmarshal([]byte(commandConfig), &baseConfig)
	if commandType == "" {
		if baseConfig.CommandType == "" {
			logger.LogError("commandType not provided", nil)
			return missingCommandTypeExitCode
		}
		commandType = baseConfig.CommandType
	}
	if repositoryPath == "" {
		if baseConfig.Path == "" {
			logger.LogError("repositoryPath not provided", nil)
			return missingRepositoryPathExitCode
		}
		repositoryPath = baseConfig.Path
	}
	repositoryPath = hopsfsMountPoint + repositoryPath
	if updateHopsworks {
		hopsBaseUrl = fmt.Sprintf("%s/hopsworks-api/api/project/%s/git/repository/%s", remoteServerUrl, projectId, repositoryId)
		logger.LogInfo(fmt.Sprintf("Remote server repository url set to %s", hopsBaseUrl), nil)
	}
	return successExitCode
}

func Usage() {
	fmt.Fprintf(os.Stderr, "Usage of %s:\n", os.Args[0])
	fmt.Fprintf(os.Stderr, "  \nOptions:\n")
	flag.PrintDefaults()
}
