package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"github.com/fatih/structs"
	"os"
	"testing"
)

func TestMain_CLIArgs(t *testing.T) {
	updateHopsworks = false
	repoDir := "test_dir/hops-examples"
	remoteUrl := "https://github.com/logicalclocks/hops-examples.git"
	oldArgs := os.Args
	defer func() { os.Args = oldArgs }()

	cases := []struct {
		Name         string
		Args         []string
		ExpectedExit int
	}{
		{
			"invalid command json string",
			[]string{"-commandConfig", "test"},
			invalidCommandJsonStringExitCode,
		},
		{
			"commandType not set",
			[]string{"-commandConfig",
				`{"app":"Hopsworks"}`},
			missingCommandTypeExitCode,
		},
		{
			"repositoryPath not set",
			[]string{"-commandConfig", CommandJsonString(structs.Map(&CloneCommandConfiguration{Url: remoteUrl})), "-commandType", "clone"},
			missingRepositoryPathExitCode,
		},
		{
			"clone",
			[]string{"-repositoryPath", repoDir, "-commandType", Clone, "-commandConfig", CommandJsonString(structs.Map(&CloneCommandConfiguration{Url: remoteUrl}))},
			successExitCode,
		},
		{
			"add_remote",
			[]string{"-repositoryPath", repoDir, "-commandType", AddRemote, "-commandConfig", CommandJsonString(structs.Map(&AddRemoteCommandConfiguration{RemoteName: "test_remote", RemoteURL: remoteUrl}))},
			successExitCode,
		},
		{
			"delete_remote",
			[]string{"-repositoryPath", repoDir, "-commandType", DeleteRemote, "-commandConfig", CommandJsonString(structs.Map(&DeleteRemoteCommandConfiguration{RemoteName: "test_remote"}))},
			successExitCode,
		},
		{
			"status",
			[]string{"-repositoryPath", repoDir, "-commandType", Status, "-commandConfig", `{}`},
			successExitCode,
		},
		{
			"create_branch",
			[]string{"-repositoryPath", repoDir, "-commandType", CreateBranch, "-commandConfig", CommandJsonString(structs.Map(&CreateBranchCommandConfiguration{BranchName: "test_branch"}))},
			successExitCode,
		},
		{
			"delete_branch",
			[]string{"-repositoryPath", repoDir, "-commandType", DeleteBranch, "-commandConfig", CommandJsonString(structs.Map(&DeleteBranchCommandConfiguration{BranchName: "test_branch"}))},
			successExitCode,
		},
		{
			"create_checkout_branch",
			[]string{"-repositoryPath", repoDir, "-commandType", CreateBranch, "-commandConfig", CommandJsonString(structs.Map(&CreateBranchCommandConfiguration{BranchName: "test_branch2", Checkout: true}))},
			successExitCode,
		},
		{
			"commit",
			[]string{"-repositoryPath", repoDir, "-commandType", Commit, "-commandConfig", CommandJsonString(structs.Map(&CommitCommandConfiguration{Message: "Test commit", All: true, CommitterSignature: CommitterSignature{Name: "Admin", Email: "admin@hopsworks.ai"}}))},
			successExitCode,
		},
		{
			"pull",
			[]string{"-repositoryPath", repoDir, "-commandType", Pull, "-commandConfig", CommandJsonString(structs.Map(&PullCommandConfiguration{RemoteName: "origin", BranchName: "master"}))},
			successExitCode,
		},
	}

	for _, tc := range cases {
		t.Run(fmt.Sprintf("%s args %v", tc.Name, tc.Args), func(t *testing.T) {
			// this call is required because otherwise flags panics,
			// if args are set between flag.Parse call
			flag.CommandLine = flag.NewFlagSet(tc.Name, flag.ExitOnError)
			// we need a value to set Args[0] to cause flag begins parsing at Args[1]
			os.Args = append([]string{tc.Name}, tc.Args...)
			if tc.Name == Commit {
				ExecuteOnPath(repoDir, "touch test.txt")
			}
			actualExit := ExecuteCommand()
			if tc.ExpectedExit != actualExit {
				t.Errorf("Wrong exit code for test: %v,  args: %v, expected: %v, got: %v",
					tc.Name, tc.Args, tc.ExpectedExit, actualExit)
			}
		})
	}
}

func CommandJsonString(config map[string]interface{}) string {
	jsonString, err := json.Marshal(config)
	if err != nil {
		return ""
	}
	return string(jsonString)
}
