package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/go-git/go-git/v5"
	"github.com/go-git/go-git/v5/plumbing"
	plumbing_object "github.com/go-git/go-git/v5/plumbing/object"
	git_http "github.com/go-git/go-git/v5/plumbing/transport/http"
	"strings"
	"time"
)

type RepositoryActions struct {
	r *git.Repository
}

func (repo *RepositoryActions) ExecuteRepositoryAction() (err error) {
	message := fmt.Sprintf("%s was successfull", commandType)

	switch commandType {
	case Commit:
		var config CommitCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		_, _, err = repo.Commit(config)
	case Pull:
		var config PullCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		err = repo.Pull(config)
	case Push:
		var config PushCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		err = repo.Push(config)
	case Status:
		statusDTO, err := repo.Status()
		if err == nil {
			jsonStr, _ := json.Marshal(statusDTO)
			message = string(jsonStr)
		}
	default:
		err = errors.New("Unknown repository action: " + commandType)
	}

	if updateHopsworks {
		finalState := SuccessState
		if err != nil {
			finalState = FailedState
			message = fmt.Sprintf("%s failed. %s", commandType, err.Error())
			logger.LogError(err.Error(), nil)
		} else {
			logger.LogInfo(message, nil)
			UpdateBranchCommits(repo.r)
		}
		UpdateFinalRepositoryState(repo.r, message, finalState)
	}
	return err
}

func (repo *RepositoryActions) Push(pushCommandConfig PushCommandConfiguration) error {
	err := repo.r.Push(&git.PushOptions{
		RemoteName: pushCommandConfig.RemoteName,
		Auth: &git_http.BasicAuth{
			Username: gitUsername,
			Password: gitToken,
		},
		Force: pushCommandConfig.Force,
	})
	return err
}

func (repo *RepositoryActions) Pull(pullCommandConfig PullCommandConfiguration) error {
	currentBranch, _, err := repo.GetRepositoryBranchAndCommit()

	branchName := pullCommandConfig.BranchName
	remoteName := pullCommandConfig.RemoteName
	committer := pullCommandConfig.CommitterSignature

	fetchOptions := &git.FetchOptions{
		Auth: &git_http.BasicAuth{
			Username: gitUsername,
			Password: gitToken,
		},
		Force: pullCommandConfig.Force}

	// Fetch all remotes
	err = repo.r.Fetch(fetchOptions)
	if err != nil && err.Error() != "already up-to-date" {
		return err
	}
	//set config
	err = SetRepositoryUserConfig(committer.Name, committer.Email)
	if err != nil {
		return err
	}
	//Apply rebase
	cmd := "git rebase "
	if branchName != "" && remoteName != "" {
		cmd = cmd + remoteName + "/" + branchName + " " + currentBranch
		logger.LogInfo(fmt.Sprintf("Applying git rebase:  `%s`", cmd), nil)
	} else {
		err = errors.New("provide branch and origin")
		logger.LogError(err.Error(), nil)
		return err
	}

	if err = ExecuteOnPath(repositoryPath, cmd); err != nil && err.Error() != "already up-to-date" {
		logger.LogError(err.Error(), nil)
		logger.LogInfo("Aborting rebase", nil)
		ExecuteOnPath(repositoryPath, "git rebase --abort") //Noted if an error occurs the HEAD is detached
		return errors.New(err.Error() + " " + cmd)
	}
	return nil
}

func CloneRepo(path string, cloneCommandConfig CloneCommandConfiguration) (*git.Repository, error) {
	url := cloneCommandConfig.Url
	logger.LogInfo(fmt.Sprintf("Cloning at path: %s ", path), nil)
	gitCloneOptions := &git.CloneOptions{
		URL:               url,
		RecurseSubmodules: git.DefaultSubmoduleRecursionDepth,
		SingleBranch:      false,
	}
	if cloneCommandConfig.Branch != "" {
		gitCloneOptions.SingleBranch = true
		gitCloneOptions.ReferenceName = plumbing.ReferenceName("refs/heads/" + cloneCommandConfig.Branch)
	}
	if gitUsername != "" && gitToken != "" {
		gitCloneOptions.Auth = &git_http.BasicAuth{
			Username: gitUsername,
			Password: gitToken,
		}
	}
	r, err := git.PlainClone(path, false, gitCloneOptions)
	if updateHopsworks {
		finalState := SuccessState
		message := "Repository successfully cloned"
		if err != nil {
			finalState = FailedState
			message = fmt.Sprintf("Failed to clone branch. %s", err.Error())
			SendFinalStateToHopsworks("", "", message, finalState)
		} else {
			UpdateBranchCommits(r)
			UpdateFinalRepositoryState(r, message, finalState)
		}
	}
	return r, err
}

func (repo *RepositoryActions) GetRepositoryBranchAndCommit() (branchName string, commitHash string, err error) {
	// ... retrieving the branch being pointed by HEAD
	ref, err := repo.r.Head()
	if err != nil {
		return branchName, commitHash, err
	}
	branchName = strings.ReplaceAll(ref.Name().String(), "refs/heads/", "")
	logger.LogInfo(fmt.Sprintf("Current branch name is %s", branchName), nil)
	// ... retrieving the commit object
	commit, err := repo.r.CommitObject(ref.Hash())
	if err != nil {
		return branchName, commitHash, err
	}
	commitHash = commit.Hash.String()
	logger.LogInfo(fmt.Sprintf("Current commit is %s", commitHash), nil)
	return branchName, commitHash, nil
}

func (repo *RepositoryActions) GetCommits() (commitsDTOs []CommitDTO, err error) {
	// List the history of the repository
	logger.LogInfo("git logs --oneline", nil)
	commitIter, err := repo.r.Log(&git.LogOptions{All: true})
	if err != nil {
		logger.LogError(fmt.Sprintf("Failed to get commits. %s", err.Error()), nil)
		return commitsDTOs, err
	}
	count := 0
	for {
		commit, err := commitIter.Next()
		if err != nil || count >= 20 {
			break
		}
		commitDTO := CommitDTO{
			Name:       commit.Committer.Name,
			Email:      commit.Committer.Email,
			Time:       commit.Committer.When,
			Message:    commit.Message,
			CommitHash: commit.Hash.String(),
		}

		commitsDTOs = append(commitsDTOs, commitDTO)
		count++
	}
	return commitsDTOs, nil
}

func (repo *RepositoryActions) Status() (statusDTO *StatusDTO, err error) {
	w, err := repo.r.Worktree()
	if err != nil {
		return statusDTO, err
	}
	// Check the statusDTO.
	logger.LogInfo("git statusDTO --porcelain", nil)
	s, err := w.Status()
	if err != nil {
		return statusDTO, nil
	}
	var files []FileStatus
	for path, status := range s {
		files = append(files, FileStatus{
			File:   path,
			Status: string(status.Worktree),
			Extra:  status.Extra,
		})
	}
	statusDTO = &StatusDTO{Files: files}
	return statusDTO, nil
}

func (repo *RepositoryActions) Commit(commitCommandConfig CommitCommandConfiguration) (status git.Status, commit plumbing.Hash, err error) {
	w, err := repo.r.Worktree()
	if err != nil {
		return status, commit, err
	}
	// Check the status.
	logger.LogInfo("git status --porcelain", nil)
	status, err = w.Status()
	if err != nil {
		return status, commit, err
	}
	logger.LogInfo(fmt.Sprintf("Current Status is %s", status), nil)
	if status.IsClean() {
		err := errors.New("nothing to commit, working tree clean")
		logger.LogError(err.Error(), nil)
		return status, commit, err
	}

	if commitCommandConfig.All {
		//Get each file in status map and add it separately.
		for path, _ := range status {
			logger.LogInfo(fmt.Sprintf("Adding path %s", path), nil)
			w.Add(path)
		}
	} else {
		//Only add the files specified
		for _, path := range commitCommandConfig.Files {
			logger.LogInfo(fmt.Sprintf("Adding path %s", path), nil)
			w.Add(path)
		}
	}

	//check new status
	status, err = w.Status()
	if err != nil {
		logger.LogError(err.Error(), nil)
		return status, commit, err
	}
	logger.LogInfo(fmt.Sprintf("Current Status after add is %s", status), nil)

	commit, err = w.Commit(commitCommandConfig.Message, &git.CommitOptions{
		Author: &plumbing_object.Signature{
			Name:  commitCommandConfig.CommitterSignature.Name,
			Email: commitCommandConfig.CommitterSignature.Email,
			When:  time.Now(),
		},
	})
	if err != nil {
		logger.LogError(err.Error(), nil)
		//We should unstage all the files in case the commit failed
		cmd := "git reset HEAD -- ."
		ExecuteOnPath(repositoryPath, cmd)
	}
	//get new status
	status, _ = w.Status()
	logger.LogInfo(fmt.Sprintf("Added commit is %s", commit), nil)
	logger.LogInfo(fmt.Sprintf("Current commit hash is %s", commit), nil)
	return status, commit, err
}

func SetRepositoryUserConfig(user string, email string) error {
	cmd := "git config user.name " + "\"" + user + "\""
	logger.LogInfo(fmt.Sprintf("Set git user.name config command: %s", cmd), nil)
	err := ExecuteOnPath(repositoryPath, cmd)
	if err != nil {
		return err
	}
	cmd = "git config user.email " + "\"" + email + "\""
	logger.LogInfo(fmt.Sprintf("Set git user.email config command: %s", cmd), nil)
	return ExecuteOnPath(repositoryPath, cmd)
}

func OpenRepo(repositoryPath string) (*git.Repository, error) {
	logger.LogInfo("Opening repository on path "+repositoryPath, nil)
	repository, err := git.PlainOpen(repositoryPath)
	if err != nil {
		message := fmt.Sprintf("%s failed. %s", commandType, err.Error())
		logger.LogError(message, nil)
		//update hopsworks
		if updateHopsworks {
			SendFinalStateToHopsworks("", "", message, FailedState)
		}
	}
	return repository, err
}
