package main

import (
	"fmt"
	"github.com/stretchr/testify/require"
	"testing"
)

var TestRepoUrl = "https://github.com/logicalclocks/hopsgit-testing.git"

func TestClone(t *testing.T) {
	r, _, err := CloneRepoTest(t, "clone")
	remotes, err := r.Remotes()
	require.Equal(t, err, nil)
	require.Equal(t, len(remotes), 1)
}

func TestOpenRepo(t *testing.T) {
	_, repoDir, err := CloneRepoTest(t, "open_repo")
	r2, err := OpenRepo(repoDir)
	require.Equal(t, err, nil)
	require.NotNil(t, r2)
}

func TestRepositoryActions_Status(t *testing.T) {
	r, repoPath, _ := CloneRepoTest(t, "status")
	repoActions := &RepositoryActions{r: r}
	status, err := repoActions.Status()
	require.Nil(t, err)
	require.Equal(t, len(status.Files), 0)
	// add some file
	err = ExecuteOnPath(repoPath, "touch test.txt")
	require.Nil(t, err)
	status, err = repoActions.Status()
	logger.LogInfo(fmt.Sprintf("%s", status), nil)
	require.Nil(t, err)
	require.EqualValues(t, len(status.Files), 1)
	require.Contains(t, status.Files, FileStatus{File: "test.txt", Status: "?", Extra: ""})
}

func TestRepositoryActions_GetCommits(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "repo_commits")
	repoActions := &RepositoryActions{r: r}
	commits, err := repoActions.GetCommits()
	require.Nil(t, err)
	require.Greater(t, len(commits), 0)
}

func TestRepositoryActions_GetRepositoryBranchAndCommit(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "test_get_branch_and_commit")
	repoActions := &RepositoryActions{r: r}
	branch, commit, err := repoActions.GetRepositoryBranchAndCommit()
	require.Nil(t, err)
	require.Equal(t, branch, "main")
	require.Equal(t, commit, "88b0817206063bdeb0cfeaa8b092ce7290ca4a47")
}

func TestRepositoryActions_Commit(t *testing.T) {
	r, repoPath, _ := CloneRepoTest(t, "commit")
	repoActions := &RepositoryActions{r: r}
	// add some file
	err := ExecuteOnPath(repoPath, "touch test.txt")
	require.Nil(t, err)
	status, commitHash, err := repoActions.Commit(CommitCommandConfiguration{Message: "Test commit", All: true})
	require.Nil(t, err)
	logger.LogInfo(status.String(), nil)
	require.Equal(t, status.IsClean(), true)
	_, commit, err := repoActions.GetRepositoryBranchAndCommit()
	require.Nil(t, err)
	require.Equal(t, commitHash.String(), commit)
}

func TestRepositoryActions_Pull(t *testing.T) {
	repoDir, err := createRepoDir("pull")
	require.Nil(t, err)
	r, err := CloneRepo(repoDir, CloneCommandConfiguration{Url: "https://github.com/gibchikafa/hops-examples.git"})
	repositoryPath = repoDir
	require.Nil(t, err)
	require.NotNil(t, r)
	//checkout to some release branch on remote
	branchActions := &BranchActions{repository: r}
	err = branchActions.Checkout("test_git", "", true)
	require.Nil(t, err)
	repoActions := &RepositoryActions{r: r}
	//pull from master
	err = repoActions.Pull(PullCommandConfiguration{
		RemoteName: "origin",
		BranchName: "master",
		CommitterSignature: CommitterSignature{
			Name:  "Admin",
			Email: "admin@hopsworks.ai",
		},
	})
	require.Nil(t, err)
}
